/*
 * Created on October 21, 2005
 *
 */
package com.sunlabs.archsim.fleet.ships.duplicator;
import com.sunlabs.archsim.base.common.Parameters;

import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;


/**
 * @author mh193489
 * 
 * Duplicator ship for fleet.
 * Accepts integer on In terminal and outputs the cumulative sum on Out terminal.
 */
public class Duplicator extends Component {
		
	InputTerminal in;
	OutputTerminal ackIn;
 
	OutputTerminal out1;
	InputTerminal ackOut1;
	OutputTerminal out2;
	InputTerminal ackOut2;
        
	/** accumulator action hidden in the ship */
	DuplicatorAction duplicatorAction;
	
	public Duplicator(String name, Parameters params) {
        super(name, null);
        
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
		
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");					
		
			} // for
		} // if params != null
		        
        construct(name);       
        
	} // Duplicator

	public Duplicator(String name) {
    	super(name, null);
        construct(name);				
	} // Duplicator

	
    private void construct(String name) {
	
        // create terminals for the Duplicator component
		in= new InputTerminal(DuplicatorDefaults.IN, this);
        //in.beNonResetting();
        ackIn= new OutputTerminal(DuplicatorDefaults.ACK_IN, this);

        out1= new OutputTerminal(DuplicatorDefaults.OUT1, this); 
        ackOut1= new InputTerminal(DuplicatorDefaults.ACK_OUT1, this);
        out2= new OutputTerminal(DuplicatorDefaults.OUT2, this); 
        ackOut2= new InputTerminal(DuplicatorDefaults.ACK_OUT2, this);

	            // create the action performing the adding
		duplicatorAction= new DuplicatorAction(DuplicatorDefaults.ADDER, this);
	
        // attach terminals to the action
        duplicatorAction.attach(in);
		duplicatorAction.attach(ackIn);
		duplicatorAction.attach(out1);
		duplicatorAction.attach(ackOut1);		
        duplicatorAction.attach(out2);
		duplicatorAction.attach(ackOut2);
    }
        

	
	/** 
	 * Get a terminal by name.
	 * Return null if no such terminal exists.
	 */
	public Terminal getTerminal(String name) {
		if (name.equals(in.getBaseName()) 
				|| name.equals(in.getBaseAlias()))
			return in;
		else if (name.equals(ackIn.getBaseName()) 
				|| name.equals(ackIn.getBaseAlias()))
			return ackIn;
                
		else if (name.equals(out1.getBaseName())
				|| name.equals(out1.getBaseAlias()))
			return out1;
		else if (name.equals(ackOut1.getBaseName())
				|| name.equals(ackOut1.getBaseAlias()))
			return ackOut1;
                
		else if (name.equals(out2.getBaseName())
				|| name.equals(out2.getBaseAlias()))
			return out2;
		else if (name.equals(ackOut2.getBaseName())
				|| name.equals(ackOut2.getBaseAlias()))
			return ackOut2;
		else 
			return null;
			
	} // getTerminal
	
} // Duplicator;


