/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.ships.fetchAndIssue;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.fleet.codeStorage.CodeBag;
import com.sunlabs.archsim.fleet.codeStorage.CodeStorage;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.fleet.parser.program.ProgramParser;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSink;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.runner.Runner;

import junit.framework.TestCase;
import junit.textui.TestRunner;

/**
 * @author ib27688
 *
 */
public class FetchAndIssueTest extends TestCase {

	private LinkedList<Payload> referenceInstructionData= new LinkedList<Payload>();
	private LinkedList<Payload> referenceLiteralData= new LinkedList<Payload>();
	
	public static void main(String[] args) {
		TestRunner.run(FetchAndIssueTest.class);
		// FetchAndIssueTest test= new FetchAndIssueTest();
		// test.setUp();
		//test.testOperation();
		//test.tearDown();
	}

	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() {
		
		// populate code storage
		// need path to the input file/program
		String path= "src/" +
			FetchAndIssueTestSource.class.getPackage().getName().replace('.','/') + "/";
		
		// make a parser and parse the input file
		ProgramParser parser= new ProgramParser();
		try {
			parser.parse(path+"FetchIssueTestCode.fleet");
		}
		catch (Exception e) {
			fail("FetchAndIssueTest setUp could not populate code storage from file "
					+ path+"FetchIssueTestCode.fleet. " 
					+ "Exception message: " + e.getMessage());
		}
		
		// build reference lists. First the bootstrap codebags.
		CodeStorage codeStorage= CodeStorage.getInstance();
		for (CodeBag bag : codeStorage.bootstrap) {
			for (Payload p : bag.instructions()) {
				if (p.repeats ==1 || p.repeats < 0) {
					// classify the instruction
					if (p.type == Payload.PayloadType.INSTRUCTION) {
						referenceInstructionData.add(p);
					}
					else if (p.type == Payload.PayloadType.LITERAL) {
						referenceLiteralData.add(p);
					}
				}
				else if (p.repeats > 1) {
					// need to take care of cloning if data to make sure this always works
					// it will be OK for Integers and Strings, but if Data.value does not
					// implement a deep clone, two payloads may carry the same data object
					for (int i=0; i< p.repeats; i++) {
						if (p.type == Payload.PayloadType.INSTRUCTION) {
							referenceInstructionData.add((Payload)p.clone());
						}
						else if (p.type == Payload.PayloadType.LITERAL) {
							referenceLiteralData.add((Payload)p.clone());
						}
					}
				} // else		
			} // for Payload 
		} // for CodeBag
		
		// continue building the reference list. Now all codebags
		Iterator<String> bni= codeStorage.bagNameIterator();
		while (bni.hasNext()) {
			CodeBag bag= codeStorage.getBag(bni.next());
			for (Payload p : bag.instructions()) {
				if (p.repeats ==1 || p.repeats < 0) {
					// classify the instruction
					if (p.type == Payload.PayloadType.INSTRUCTION) {
						referenceInstructionData.add(p);
					}
					else if (p.type == Payload.PayloadType.LITERAL) {
						referenceLiteralData.add(p);
					}
				}
				else if (p.repeats > 1) {
					// need to take care of cloning if data to make sure this always works
					// it will be OK for Integers and Strings, but if Data.value does not
					// implement a deep clone, two payloads may carry the same data object
					for (int i=0; i< p.repeats; i++) {
						if (p.type == Payload.PayloadType.INSTRUCTION) {
							referenceInstructionData.add((Payload)p.clone());
						}
						else if (p.type == Payload.PayloadType.LITERAL) {
							referenceLiteralData.add((Payload)p.clone());
						}
					}
				} // else
			} // for Payload 
		} // for CodeBag		

	} // setUp

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() {
	}

	
	


	// now test the operation of fetch and issue
	public void testOperation() {
		OperationLoader ldr= new OperationLoader();
		List ops= ldr.load("src/" 
				+ FetchAndIssueTest.class.getPackage().getName().replace('.', '/') 
				+ "/FetchAndIssueTestScript.xml");
		
		Runner r= new Runner();
		r.perform(ops);
		
		// get the data from the sink
		PayloadSink instructionSink= (PayloadSink)(Entities.find("InstructionSink"));

		assertEquals(referenceInstructionData.size(), instructionSink.receivedData.size());
		boolean match= true;
		for (Payload p :instructionSink.receivedData ) {
			match= match &&  referenceInstructionData.remove(p);
		}
		assertTrue(match);
		
		//assertTrue(referenceInstructionData.equals(instructionSink.receivedData));

		PayloadSink literalSink= (PayloadSink)(Entities.find("LiteralSink"));

		
		// System.out.println("reference literal size: " + referenceLiteralData.size() +  "\n\n" + 
		//		referenceLiteralData);
		
		// System.out.println("instruction sink size: " + literalSink.receivedData.size() +  "\n\n" + 
		//		literalSink.receivedData);

		
		
		assertEquals(referenceLiteralData.size(), literalSink.receivedData.size());
		match= true;
		for (Payload p :literalSink.receivedData ) {
			match= match &&  referenceLiteralData.remove(p);
		}
		assertTrue(match);

		
		// assertTrue(referenceLiteralData.equals(literalSink.receivedData));
		
	} // operationTest	
	
	public void testSelfCheck() {
		Object tst=Entities.find("FetchAndIssueTest");
		CompositeEntity testGroup= (CompositeEntity)tst;
		assert(testGroup.selfCheck());		
	}
	
	
} // class FetchAndIssueTest
